#!/usr/bin/env python
# -*- coding: latin-1 -*-

"""Crawl through active AWS accounts in an organization using master assumed role."""
import argparse
import boto3
from botocore import exceptions
from aws_authenticator import AWSAuthenticator as awsauth


__veersion__ = '1.0.1'


def list_accounts(
    session
) -> list:
    """List all AWS accounts in the organization."""
    print("Getting accounts...")

    client = session.client('organizations')

    paginator = client.get_paginator('list_accounts')
    accounts = []
    for page in paginator.paginate():
        for account in page['Accounts']:
            if account['Status'] == 'ACTIVE':
                accounts.append(account)
    
    print(f"Found {len(accounts)} active accounts...")

    return accounts


def get_credentials(
    session,
    assumed_role_arn: str,
    external_id: str
) -> dict:
    """Get AWS assumed role credentials with STS."""
    client = session.client('sts')

    if external_id is not None:
        sts_r = client.assume_role(
            RoleArn=assumed_role_arn,
            RoleSessionName="aws-org-crawler",
            ExternalId=external_id
        )

    else:
        sts_r = client.assume_role(
            RoleArn=assumed_role_arn,
            RoleSessionName="aws-org-crawler"
        )

    return {
        'aws_access_key_id': sts_r["Credentials"]["AccessKeyId"],
        'aws_secret_access_key': sts_r["Credentials"]["SecretAccessKey"],
        'aws_session_token': sts_r["Credentials"]["SessionToken"]
    }


def main():
    """Execute test function with SSO login."""
    # Get and parse command line arguments.
    myparser = argparse.ArgumentParser(
        add_help=True,
        allow_abbrev=False,
        description="Get STS credentials for AWS accounts in an organization",
        usage="%(prog)s [options]"
    )
    myparser.add_argument(
        "-V",
        "--version",
        action="version",
        version="%(prog)s 1.0.1"
    )
    myparser.add_argument(
        "-u",
        "--sso_url",
        action="store",
        help="[REQUIRED] AWS SSO URL",
        required=True,
        type=str
    )
    myparser.add_argument(
        "-a",
        "--account",
        action="store",
        help="[REQUIRED] AWS SSO Account ID",
        required=True,
        type=str
    )
    myparser.add_argument(
        "-r",
        "--sso_role",
        action="store",
        help="[REQUIRED] AWS SSO Role Name",
        required=False,
        default="AWSViewOnlyAccess",
        type=str
    )
    myparser.add_argument(
        "-n",
        "--assumed_role",
        action="store",
        help="[OPTIONAL: default = AWSViewOnlyAccess] IAM Assumed Role Name",
        required=False,
        default="AWSViewOnlyAccess",
        type=str
    )
    myparser.add_argument(
        "-e",
        "--external_id",
        action="store",
        help="[OPTIONAL: default = None] AWS External ID",
        required=False,
        default=None,
        type=str
    )
    myparser.add_argument(
        "-g",
        "--region",
        action="store",
        help="[OPTIONAL: default = us-east-1] AWS Region Name",
        required=False,
        default="us-east-1",
        type=str
    )
    args = myparser.parse_args()
    sso_url = args.sso_url
    sso_role_name = args.sso_role
    sso_account_id = args.account
    assumed_role_name = args.assumed_role
    external_id = args.external_id
    region = args.region

    # Login to AWS.
    auth = awsauth(
        sso_url=sso_url,
        sso_role_name=sso_role_name,
        sso_account_id=sso_account_id,
    )
    session = auth.sso()

    # Get account list.
    accounts = list_accounts(session)
    account_ids = [account['Id'] for account in accounts]

    # Crawl through each account.
    for account_id in account_ids:
        print(f"Working on {account_id}...")

        try:
            credentials = get_credentials(
                session,
                f'arn:aws:iam::{account_id}:role/{assumed_role_name}',
                external_id
            )

            client = boto3.client(
                'sts',
                aws_access_key_id=credentials['aws_access_key_id'],
                aws_secret_access_key=credentials['aws_secret_access_key'],
                aws_session_token=credentials['aws_session_token'],
                region_name=region
            )

            response = client.get_caller_identity()['UserId']
        
        except exceptions.ClientError as e:
            response = 'Could not assume role'
        
        print(response)


if __name__ == '__main__':
    main()
